"""
元数据处理模块
提供元数据管理、查询、图谱分析和非结构化数据处理的核心功能
"""

import time
import logging
from app.services.neo4j_driver import neo4j_driver
import ast
import re
import pandas as pd
from minio import S3Error
from py2neo import Relationship
import json
import io
import random
import string
import numpy as np
from openai import OpenAI

logger = logging.getLogger("app")

# LLM客户端配置
api_key = "sk-86d4622141d74e9a8d7c38ee873c4d91"
base_url = "https://dashscope.aliyuncs.com/compatible-mode/v1"
model_name = "qwen-turbo"

def get_formatted_time():
    """获取格式化的当前时间"""
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())

def translate_and_parse(data):
    """转换并解析数据"""
    if isinstance(data, dict):
        return data
    else:
        return {}

# LLM服务
def llm_client(content):
    """调用LLM服务进行内容生成"""
    client = OpenAI(
        api_key=api_key,
        base_url=base_url
    )

    try:
        completion = client.chat.completions.create(
            model=model_name,
            messages=[
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": content}
            ]
        )
        return completion.choices[0].message.content.strip()
    except Exception as e:
        logger.error(f"LLM调用失败: {str(e)}")
        return None

def infer_column_type(df):
    """推断DataFrame的列类型"""
    column_types = {}
    for column in df.columns:
        if df[column].dtype == 'object':
            # 如果列是对象类型，尝试判断是否为日期或字符串
            if pd.to_datetime(df[column], errors='coerce').notna().all():
                column_types[column] = 'datetime'
            else:
                column_types[column] = 'varchar(255)'
        elif pd.api.types.is_integer_dtype(df[column]):
            column_types[column] = 'int'
        elif pd.api.types.is_float_dtype(df[column]):
            column_types[column] = 'float'
        elif pd.api.types.is_bool_dtype(df[column]):
            column_types[column] = 'boolean'
        else:
            column_types[column] = 'varchar(255)'
    return column_types

def meta_list(page, page_size, search="", en_name_filter=None, 
             name_filter=None, category_filter=None, time_filter=None, tag_filter=None):
    """
    获取元数据列表
    
    Args:
        page: 当前页码
        page_size: 每页数量
        search: 搜索关键词
        en_name_filter: 英文名称过滤
        name_filter: 名称过滤
        category_filter: 分类过滤
        time_filter: 时间过滤
        tag_filter: 标签过滤
    
    Returns:
        tuple: (result_list, total_count)
    """
    try:
        with neo4j_driver.get_session() as session:
            # 构建查询条件
            match_clause = "MATCH (n:Metadata)"
            where_conditions = []
            
            if search:
                where_conditions.append(f"n.name CONTAINS '{search}'")
            
            if en_name_filter:
                where_conditions.append(f"n.en_name CONTAINS '{en_name_filter}'")
                
            if name_filter:
                where_conditions.append(f"n.name CONTAINS '{name_filter}'")
                
            if category_filter:
                where_conditions.append(f"n.category = '{category_filter}'")
                
            if time_filter:
                where_conditions.append(f"n.createTime CONTAINS '{time_filter}'")
                
            if tag_filter:
                where_conditions.append(f"n.tag = '{tag_filter}'")
            
            where_clause = " WHERE " + " AND ".join(where_conditions) if where_conditions else ""
            
            # 计算总数
            count_cypher = f"{match_clause}{where_clause} RETURN count(n) as count"
            count_result = session.run(count_cypher)
            total_count = count_result.single()["count"]
            
            # 分页查询
            skip = (page - 1) * page_size
            cypher = f"""
            {match_clause}{where_clause}
            RETURN n
            ORDER BY n.name
            SKIP {skip} LIMIT {page_size}
            """
            result = session.run(cypher)
            
            # 格式化结果
            result_list = []
            for record in result:
                node = dict(record["n"])
                node["id"] = record["n"].id
                result_list.append(node)
                
            return result_list, total_count
    except Exception as e:
        logger.error(f"获取元数据列表失败: {str(e)}")
        raise

def handle_id_unstructured(node_id):
    """处理非结构化数据节点"""
    try:
        with neo4j_driver.get_session() as session:
            query = "MATCH (n) WHERE id(n) = $node_id RETURN n"
            result = session.run(query, node_id=int(node_id))
            node = result.single()
            if node:
                return dict(node["n"])
            else:
                return None
    except Exception as e:
        logger.error(f"处理非结构化数据节点失败: {str(e)}")
        raise

def get_file_content(minio_client, bucket_name, object_name):
    """从MinIO获取文件内容"""
    try:
        # 获取对象
        response = minio_client.get_object(bucket_name, object_name)
        
        # 读取内容
        file_content = response.read().decode('utf-8')
        return file_content
    except S3Error as e:
        logger.error(f"MinIO访问失败: {str(e)}")
        raise
    finally:
        response.close()
        response.release_conn()

def parse_text(text):
    """解析文本内容，提取关键信息"""
    # 提取作者信息
    author_match = re.search(r'作者[：:]\s*(.+?)[\n\r]', text)
    author = author_match.group(1) if author_match else ""
    
    # 提取关键词
    keyword_match = re.search(r'关键词[：:]\s*(.+?)[\n\r]', text)
    keywords = keyword_match.group(1) if keyword_match else ""
    
    return {
        "author": author.strip(),
        "keywords": keywords.strip()
    }

def parse_keyword(content):
    """解析关键词"""
    if "，" in content:
        return content.split("，")
    elif "," in content:
        return content.split(",")
    elif "、" in content:
        return content.split("、")
    else:
        return [content]

def text_resource_solve(receiver, name, keyword):
    """处理文本资源解析"""
    try:
        # 构建提示词
        prompt = f"""将以下中文内容翻译成英文，要求:
        1. 保持原意，语法正确，符合英文表达习惯
        2. 专业术语保持精准
        中文内容: {name}
        """
        
        # 调用LLM获取英文翻译
        english_name = llm_client(prompt)
        
        # 提取关键词
        keywords = parse_keyword(keyword)
        
        # 为每个关键词获取英文翻译
        keywords_en = []
        for kw in keywords:
            prompt = f"将以下中文专业术语翻译成英文: {kw}"
            kw_en = llm_client(prompt)
            keywords_en.append(kw_en)
            
        # 构建返回数据
        return {
            "name": name,
            "en_name": english_name,
            "keywords": keywords,
            "keywords_en": keywords_en
        }
    except Exception as e:
        logger.error(f"文本资源处理失败: {str(e)}")
        raise

def meta_kinship_graph(node_id):
    """
    获取元数据亲缘关系图谱
    
    Args:
        node_id: 元数据节点ID
    
    Returns:
        dict: 图谱数据
    """
    try:
        with neo4j_driver.get_session() as session:
            # 获取节点和直接关系
            cypher = """
            MATCH (n)-[r]-(m)
            WHERE id(n) = $node_id
            RETURN n, r, m
            """
            result = session.run(cypher, node_id=int(node_id))
            
            # 格式化结果为图谱数据
            nodes = {}
            relationships = []
            
            for record in result:
                # 处理源节点
                source_node = dict(record["n"])
                source_node["id"] = record["n"].id
                nodes[source_node["id"]] = source_node
                
                # 处理目标节点
                target_node = dict(record["m"])
                target_node["id"] = record["m"].id
                nodes[target_node["id"]] = target_node
                
                # 处理关系
                rel = record["r"]
                relationship = {
                    "id": rel.id,
                    "source": record["n"].id,
                    "target": record["m"].id,
                    "type": rel.type
                }
                relationships.append(relationship)
            
            # 转换为列表
            nodes_list = list(nodes.values())
            
            return {
                "nodes": nodes_list,
                "relationships": relationships
            }
    except Exception as e:
        logger.error(f"获取元数据亲缘关系图谱失败: {str(e)}")
        raise

def meta_impact_graph(node_id):
    """
    获取元数据影响关系图谱
    
    Args:
        node_id: 元数据节点ID
    
    Returns:
        dict: 图谱数据
    """
    try:
        with neo4j_driver.get_session() as session:
            # 获取所有可达节点和关系
            cypher = """
            MATCH path = (n)-[*1..3]-(m)
            WHERE id(n) = $node_id
            RETURN path
            """
            result = session.run(cypher, node_id=int(node_id))
            
            # 格式化结果
            nodes = {}
            relationships = set()
            
            for record in result:
                path = record["path"]
                
                # 处理路径中的所有节点
                for node in path.nodes:
                    node_dict = dict(node)
                    node_dict["id"] = node.id
                    nodes[node.id] = node_dict
                
                # 处理路径中的所有关系
                for rel in path.relationships:
                    relationship = (rel.id, rel.start_node.id, rel.end_node.id, rel.type)
                    relationships.add(relationship)
            
            # 转换为列表
            nodes_list = list(nodes.values())
            relationships_list = [
                {
                    "id": rel[0],
                    "source": rel[1],
                    "target": rel[2],
                    "type": rel[3]
                }
                for rel in relationships
            ]
            
            return {
                "nodes": nodes_list,
                "relationships": relationships_list
            }
    except Exception as e:
        logger.error(f"获取元数据影响关系图谱失败: {str(e)}")
        raise

def parse_entity_relation(text):
    """从文本中解析实体关系"""
    try:
        # 构建提示词
        prompt = f"""
        请从以下文本中提取实体及其关系，以JSON格式返回，格式为:
        [
          {{"entity1": "实体1", "relation": "关系", "entity2": "实体2"}}
        ]
        
        文本内容:
        {text}
        """
        
        # 调用LLM获取关系提取结果
        result = llm_client(prompt)
        
        # 解析JSON结果
        try:
            relations = json.loads(result)
            return relations
        except json.JSONDecodeError:
            logger.error(f"关系提取结果JSON解析失败: {result}")
            return []
            
    except Exception as e:
        logger.error(f"实体关系提取失败: {str(e)}")
        return []
        
def handle_txt_graph(node_id, entity, entity_en):
    """处理文本图谱创建"""
    try:
        # 创建实体节点
        with neo4j_driver.get_session() as session:
            # 查找源节点
            query = "MATCH (n) WHERE id(n) = $node_id RETURN n"
            result = session.run(query, node_id=int(node_id))
            source_node = result.single()["n"]
            
            if not source_node:
                return False
                
            # 创建实体节点
            cypher = """
            MERGE (e:Entity {name: $name, en_name: $en_name})
            ON CREATE SET e.createTime = $create_time
            RETURN e
            """
            
            create_time = get_formatted_time()
            result = session.run(
                cypher,
                name=entity,
                en_name=entity_en,
                create_time=create_time
            )
            
            entity_node = result.single()["e"]
            
            # 创建关系
            if source_node and entity_node:
                # 检查关系是否已存在
                rel_check = """
                MATCH (s)-[r:CONTAINS]->(e) 
                WHERE id(s) = $source_id AND id(e) = $entity_id
                RETURN r
                """
                
                rel_result = session.run(
                    rel_check,
                    source_id=source_node.id,
                    entity_id=entity_node.id
                )
                
                # 如果关系不存在，则创建
                if not rel_result.single():
                    rel_create = """
                    MATCH (s), (e) 
                    WHERE id(s) = $source_id AND id(e) = $entity_id
                    CREATE (s)-[r:CONTAINS]->(e)
                    RETURN r
                    """
                    
                    session.run(
                        rel_create,
                        source_id=source_node.id,
                        entity_id=entity_node.id
                    )
                    
            return True
    except Exception as e:
        logger.error(f"文本图谱处理失败: {str(e)}")
        return False

def solve_unstructured_data(node_id, minio_client, prefix):
    """处理非结构化数据并提取实体关系"""
    try:
        # 获取节点数据
        node_data = handle_id_unstructured(node_id)
        if not node_data:
            logger.error(f"节点不存在: {node_id}")
            return False
            
        # 获取对象路径
        object_name = node_data.get('objectName')
        if not object_name:
            logger.error(f"文档路径不存在: {node_id}")
            return False
            
        # 获取文件内容
        file_content = get_file_content(minio_client, bucket_name=node_data.get('bucket_name', 'dataops'), object_name=object_name)
        
        # 解析文本内容中的实体关系
        relations = parse_entity_relation(file_content[:5000])  # 只处理前5000字符，避免过大内容
        
        # 如果成功提取了关系
        if relations:
            # 更新节点信息
            with neo4j_driver.get_session() as session:
                update_cypher = """
                MATCH (n) WHERE id(n) = $node_id
                SET n.processed = true, n.processTime = $process_time
                RETURN n
                """
                
                process_time = get_formatted_time()
                session.run(
                    update_cypher,
                    node_id=int(node_id),
                    process_time=process_time
                )
                
                # 为每个提取的关系创建实体和关系
                for relation in relations:
                    entity1 = relation.get("entity1", "")
                    relation_type = relation.get("relation", "")
                    entity2 = relation.get("entity2", "")
                    
                    if entity1 and entity2 and relation_type:
                        # 翻译实体名称为英文
                        entity1_en = llm_client(f"将以下中文专业术语翻译成英文: {entity1}")
                        entity2_en = llm_client(f"将以下中文专业术语翻译成英文: {entity2}")
                        
                        # 创建第一个实体
                        entity1_cypher = """
                        MERGE (e:Entity {name: $name})
                        ON CREATE SET e.en_name = $en_name, e.createTime = $create_time
                        RETURN e
                        """
                        
                        entity1_result = session.run(
                            entity1_cypher,
                            name=entity1,
                            en_name=entity1_en,
                            create_time=process_time
                        )
                        entity1_node = entity1_result.single()["e"]
                        
                        # 创建第二个实体
                        entity2_cypher = """
                        MERGE (e:Entity {name: $name})
                        ON CREATE SET e.en_name = $en_name, e.createTime = $create_time
                        RETURN e
                        """
                        
                        entity2_result = session.run(
                            entity2_cypher,
                            name=entity2,
                            en_name=entity2_en,
                            create_time=process_time
                        )
                        entity2_node = entity2_result.single()["e"]
                        
                        # 创建它们之间的关系
                        rel_cypher = """
                        MATCH (e1:Entity), (e2:Entity)
                        WHERE id(e1) = $entity1_id AND id(e2) = $entity2_id
                        MERGE (e1)-[r:`{relation_type}`]->(e2)
                        RETURN r
                        """.replace("{relation_type}", relation_type)
                        
                        session.run(
                            rel_cypher,
                            entity1_id=entity1_node.id,
                            entity2_id=entity2_node.id
                        )
                        
                        # 创建源节点与实体的关系
                        source_rel1_cypher = """
                        MATCH (s), (e:Entity)
                        WHERE id(s) = $source_id AND id(e) = $entity_id
                        MERGE (s)-[r:CONTAINS]->(e)
                        RETURN r
                        """
                        
                        session.run(
                            source_rel1_cypher,
                            source_id=int(node_id),
                            entity_id=entity1_node.id
                        )
                        
                        source_rel2_cypher = """
                        MATCH (s), (e:Entity)
                        WHERE id(s) = $source_id AND id(e) = $entity_id
                        MERGE (s)-[r:CONTAINS]->(e)
                        RETURN r
                        """
                        
                        session.run(
                            source_rel2_cypher,
                            source_id=int(node_id),
                            entity_id=entity2_node.id
                        )
            
            return True
        else:
            logger.warning(f"未能从文本中提取到实体关系: {node_id}")
            return False
            
    except Exception as e:
        logger.error(f"处理非结构化数据失败: {str(e)}")
        return False 